//INFO: www.rs-elc.nl
//PRBSynth1-tryout version using breadboard with Arduino UNO, no PCB needed 
// restricted version: No noise source selection, no stereo panning circuit
//RDAC on pin 13-12-11-10-09-08  (better to remove LED or LEDresistor on pin 13 of Arduino)
// using resistors 7k5-15k-30k-43k-130k-390k
// DAC weights are <-binair--> <--trinair-->
//#include <EEPROM.h>
#include <avr/pgmspace.h>//use program space to store tables
//compiled in both ARduino IDE version 1.04 and 1.8.12
// variables:
//MIDI:
byte CHAN_NR=2; //receive midi channel is 2, is set here
byte incomingByte=0;
byte statusByte=0;
byte dataByte=0;
byte statusbuffer=0;
byte NOTE_ON=144+CHAN_NR-1;//calculate note on message byte based on selected channel number
byte NOTE_OFF=128+CHAN_NR-1;
byte CONT_CHANGE=176+CHAN_NR-1;//controller
byte PROG_CHANGE=192+CHAN_NR-1;//program change 
byte PITCHW_CHANGE=224+CHAN_NR-1;//pitchbend wheel
byte AFT_CHANGE=208+CHAN_NR-1;//aftertouch
byte LAST_NOTE=0;//last note number, detected by MIDI decoder routine
byte Note; //resulting note number that is played 
byte LAST_PROG=0;//patch program nr, set local or detected by MIDI decoder routine
byte LAST_VEL=127;//key velocity value, detected by MIDI decoder routine
byte MidiVol=127;//MIDI main volume value
char LAST_PWH=0;//pitch bend wheel value, detected by MIDI decoder routine
byte LAST_AFT=0;//after touch (channelmode) value, detected by MIDI decoder routine
boolean MidiCtrl=false;//if true: disable local potmeters
//boolean MidiClk=false;//true if a MidiClock signal present, switch to ext clock mode
//boolean MidiTick=false;//true if MidiClockTick event received
//unsigned int Mclkper=200;//MIDIclock period measured, used for interpolation on generating internal tick
//unsigned int Mclktim=0;// MIDIclock period timer, used for interpolation on generating internal tick, clockx4
boolean firstbyte;// MIDI message first byte is handled
boolean KEY_DOWN=true; // key down state, high during keydown
boolean KEY_START=false; // key start state, only high at first detection of keystart
// end of MIDI variables

//Noise generator:
word Clk=200;//sampling clock divider for timer 1
word state1=1; // Pseudo Random Bit Sequence1 seed value
word state2=1;// Pseudo Random Bit Sequence2 seed value
word state3=1;
word state4=1;
byte s2cnt=0;//Pseudo Random Bit Sequence2 divider counter
byte s3cnt=0;
byte s4cnt=0;
word NoiseLen=256;//PRBS pattern length
word NoiseCnt=0;//PRBS pattern length counter
byte LenMod=0;//PRBS pattern length modulation
const word NoiseLenTab [128] PROGMEM={// PRBS pattern length scale to get 12 musical notes per octave intervals
49097,46341,43740,41285,38968,36781,34716,32768,
30929,29193,27554,26008,24548,23170,21870,20643,
19484,18390,17358,16384,15464,14596,13777,13004,
12274,11585,10935,10321,9742,9195,8679,8192,
7732,7298,6889,6502,6137,5793,5468,5161,
4871,4598,4340,4096,3866,3649,3444,3251,
3069,2896,2734,2580,2435,2299,2170,2048,
1933,1825,1722,1625,1534,1448,1367,1290,
1218,1149,1085,1024,967,912,861,813,
767,724,683,645,609,575,542,512,
483,456,431,406,384,362,342,323,
304,287,271,256,242,228,215,203,
192,181,171,161,152,144,136,128,
121,114,108,102,96,91,85,81,
76,72,68,64,60,57,54,51,
48,45,43,40,38,36,34,32
};

boolean Tick=false;//flag for update of modulators and user inteface handling
byte TickDiv=0;
char DAC=0;
char OFS=0;//offset value correction for softwarefilter
byte WNamp=48;//amplitude value setting on white noise for softwarefilter

//variables for software filter (LP/BP/HP/BR):
int LP = 0;
int HP =0;
int BP = 0;
byte F1 =50;//frequency

byte Nmode=0;//noise mode, 0=red noise, 1=pink noise, 2=white noise,  3=crisp-water, 4=crisp-crackle, 5-6-7-8-9-10-11-12-13-14-15=pattern
byte Nlim=0;//limiter value for crisp mode, set by potm noisemode

//LFO related variables and tables:
boolean E1Tr=HIGH; 
word E1cnt=0;
byte E1val=0;
byte LAST_E1val=0;
byte E1Fval=0;
byte E1mode=0;// 0=single shot 1=not used 2=LFO 3=accent 4=sweep 5=pattern 
boolean E1Accent=0;
byte Rn1=1;
byte Rncnt=0;
byte Cyclecnt=0;
byte CycleLen=8;
int Rn1_F=0;//random wave filter
//LFO rate table:wavepointer update value for each tick, rate doubles per oct for MIDI note control
const word TkUpd [128] PROGMEM={
4,	4,	4,	4,	5,	5,	5,	6,	6,	6,	7,	7,
8,	8,	9,	9,	10,	10,	11,	11,	12,	13,	14,	15,
16,	17,	18,	19,	20,	21,	22,	23,	25,	26,	28,	30,
32,	33,	35,	38,	40,	42,	45,	47,	50,	53,	56,	60,
64,	67,	71,	76,	80,	85,	90,	95,	101,	107,	113,	120,
128,	135,	143,	152,	161,	170,	180,	191,	202,	214,	227,	240,
256,	271,	287,	304,	322,	341,	361,	382,	405,	429,	454,	481,
512,	542,	574,	608,	645,	683,	723,	765,	811,	859,	909,	963,
1024,	1084,	1148,	1217,	1290,	1366,	1446,	1531,	1623,	1718,	1819,	1927,
2048,	2168,	2297,	2435,	2580,	2732,	2893,	3063,	3246,	3436,	3639,	3854,
4096,	4337,	4595,	4870,	5160,	5464,	5787,	6127
};
  
//LFO-mode beat patterns, 48x ,pattern made by changing LFO-depth in some steps, 1=accent, 0= no accent
const byte PatTab[48] PROGMEM={
B11111111,B10101010,B10101010,B10101010,B10000000,B11000000,B11100000,B11110000,
B11111000,B10100000,B10010000,B11010000,B11001000,B11011000,B00000000,B00000000,
B10000000,B11000000,B11100000,B11110000,B11111000,B11111100,B11111110,B10100000,
B10010000,B10001000,B10000100,B11010000,B11001000,B11000100,B11000010,B11101000,
B11100100,B11100010,B11110100,B11110010,B11111010,B10101000,B10100100,B10100010,
B10101100,B10101110,B10010100,B11001100,B11001100,B00000000,B00000000,B00000000
};
const byte PatLenTab[128] PROGMEM={// length of the above accent patterns:
8,8,8,8,6,6,6,6,
6,6,6,6,6,6,6,6,
8,8,8,8,8,8,8,8,
8,8,8,8,8,8,8,8,
8,8,8,8,8,8,8,8,
8,8,8,8,8,8,8,8,
// length of the sweep and random patterns:
16,12,16,16,16,16,16,16,
16,12,16,16,16,12,16,16,
8,8,8,8,8,8,8,8,
8,8,8,8,8,8,8,8,
16,16,16,16,16,16,16,16,
16,16,16,16,16,16,16,16,
16,16,16,16,16,16,16,16,
16,16,16,16,16,16,16,16,
16,16,16,16,16,16,16,16,
16,16,16,16,16,16,16,16
};

//LFO-mode sweep patterns for sweeping the LFO depth:
const byte Pat2Tab[256] PROGMEM={
255,225,200,180,160,140,120,110, 100,90,80,70,60,50,40,30,//ramp down 16
255,225,200,175,150,130,110,90, 70,50,40,30,0,0,0,0,      //ramp down 12
255,180,140,120,100,80,60,30,  255,180,140,120,100,80,60,30,//ramp down 8
255,120,60,30,255,120,60,30,   255,120,60,30,255,120,60,30,//ramp down 4
255,225,200,175,150,130,110,90, 70,50,40,30,240,120,60,30,//ramp down 12+4
255,180,140,120,100,80,60,50,  255,120,60,30,255,120,60,30,//ramp down 8+4+4
255,180,140,120,100,80,60,50,  255,120,60,30,30,70,130,255,//ramp down 8+4 ramp up 4
255,180,140,120,100,80,60,50,  30,70,130,255,30,70,130,255,//ramp down 8 ramp up 4+4
30,40,50 ,60 ,70 ,80,90,100,   110,120,140,160,180,200,225,255,//ramp up 16
30,40,50 ,70 ,90 ,110,130,150,   175,200,225,255,0,0,0,0,//ramp up 12
30,60,80,100,120,140,180,255,  30,60,80,100,120,140,180,255, //ramp up 8
30,70,130,255,30,70,130,255,   30,70,130,255,30,70,130,255,//ramp up 4
30,60,80,100,120,140,170,210,  255,210,170,140,120,100,80,30,//ramp up/down 16
30,70,100,130,180,255,190,160,  130,100,70,30,0,0,0,0,//ramp up/down 12
30,60,120,255,240,120,60,30,   30,60,120,255,240,120,60,30,//ramp up/down 8+8
30,60,120,255,240,120,60,30,   255,180,140,120,100,80,60,30//ramp up/down 8 ramp down 8

};
//end of LFO related varables and tables

// stereo pan modulator:
word E2cnt=0;
byte E2val=0;
// end of stereo pan modulator

//potmeters and trigger switch:
boolean PBtn1=HIGH; // trigger switch state
boolean LAST_PBtn1=HIGH; //previous trigger switch state
byte PotmCk=127;//sampling clock divider
byte PotmMf=0;//LFO frequency (can be set by analog in or MIDI control)
byte aPotmMf=0;// analog potmeter value set for this
byte LAST_aPotmMf=0;//previous analog potmeter value for this, check for change
byte PotmFf=0;//potm Filter freq (can be set by analog in or MIDI control)
byte aPotmFf=0;//analog potmeter valuefor this
byte LAST_aPotmFf=0;//previous analog potmeter value for this, check for change
byte PotmFt=0;
byte aPotmFt=0;
byte LAST_aPotmFt=1;
byte Ft=0;
byte LAST_Ft=0;
byte PotmNc=15;
byte aPotmNc=15;
byte LAST_aPotmNc=0;
byte PotmMa=0;
byte aPotmMa=0;
byte LAST_aPotmMa=0;
byte PotmMw=0;
byte aPotmMw=0;
byte LAST_aPotmMw=0;
byte PotmMp=0;
byte aPotmMp=0;
byte LAST_aPotmMp=0;
byte PotmPn=0;  
byte aPotmPn=0;
byte LAST_aPotmPn=0; 
byte PotmAm=127;
byte Amp=127;//output amplitude setting for filters
byte DACAmp=127;//amplitude setting for DAC

// special 6 pin parallel DAC with "8 bit" softclipping  using 6 resistors. 
// DAC has 3 binairy  + 3 trinairy  valued resistor weights (trinairy uses High/Open/Low) 
// BiTriDAC Value table with 256 entries:
const byte TDACV[256] PROGMEM= {
0,0,0,0, // 4pnts low clip level
0,0 ,0,0 ,1,1 ,0,0 ,0,0 ,1,1 ,2,2 ,2,2 ,3,3 ,0,0 ,0,0 ,1,1 ,0,0 ,0,0 ,1,1 ,2,2,// 32pnts saturation curve
2 ,3 ,4 ,4 ,5 ,4 ,4 ,5 ,6 ,6 ,7 ,
8 ,8 ,9 ,8 ,8 ,9 ,10,10,11,8 ,8 ,9 ,8 ,8 ,9 ,10,10,11,12,12,13,12,12,13,14,14,15,
16,16,17,16,16,17,18,18,19,16,16,17,16,16,17,18,18,19,20,20,21,20,20,21,22,22,23,
24,24,25,24,24,25,26,26,27,24,24,25,24,24,25,26,26,27,28,28,29,28,28,29,30,30,31,
32,32,33,32,32,33,34,34,35,32,32,33,32,32,33,34,34,35,36,36,37,36,36,37,38,38,39,
40,40,41,40,40,41,42,42,43,40,40,41,40,40,41,42,42,43,44,44,45,44,44,45,46,46,47,
48,48,49,48,48,49,50,50,51,48,48,49,48,48,49,50,50,51,52,52,53,52,52,53,54,54,55,
56,56,57,56,56,57,58,58,59,56,56, 
57,57, 56,56, 56,56, 57,57, 58,58, 58,58, 59,59, 60,60, 60,60, 61,61, 60,60, 60,60, 61,61, 62,62, 62,62, 63,63,// 32pnts saturation curve 
63,63,63,63,};// 4 pnts high clip level
//BiTriDAC Data-direction table with 256 entries:(DDR setting of port)
const byte TDACD[256] PROGMEM= {
63,63,63,63,//4 pnts low clip level
63,63, 62,62, 63,63, 61,61, 60,60, 61,61, 63,63, 62,62, 63,63, 59,59, 58,58, 59,59, 57,57, 56,56, 57,57, 59,59,// 32pnts saturation curve 
58,59,63,62,63,61,60,61,63,62,63,
63,62,63,61,60,61,63,62,63,59,58,59,57,56,57,59,58,59,63,62,63,61,60,61,63,62,63,
63,62,63,61,60,61,63,62,63,59,58,59,57,56,57,59,58,59,63,62,63,61,60,61,63,62,63,
63,62,63,61,60,61,63,62,63,59,58,59,57,56,57,59,58,59,63,62,63,61,60,61,63,62,63,
63,62,63,61,60,61,63,62,63,59,58,59,57,56,57,59,58,59,63,62,63,61,60,61,63,62,63,
63,62,63,61,60,61,63,62,63,59,58,59,57,56,57,59,58,59,63,62,63,61,60,61,63,62,63,
63,62,63,61,60,61,63,62,63,59,58,59,57,56,57,59,58,59,63,62,63,61,60,61,63,62,63,
63,62,63,61,60,61,63,62,63,59,58, 
59,59, 57,57, 56,56, 57,57, 59,59, 58,58, 59,59, 63,63, 62,62, 63,63, 61,61, 60,60, 61,61, 63,63, 62,62, 63,63,// 32pnts saturation curve 
63,63,63,63,};// 4 pnts high clip level

//Stereo panning function look-up table
const byte PanTab[32] PROGMEM={0,0,0,0, 15,30,40,50,60,70,80,90,100, 110,120, 127,127, 137,146,157,167,177,187,197,207,217,227,237,
255,255,255,255};

//LEDs:
byte LEDFm=B00000000;// 3xLED indicator for FilterMode
byte LedPWMcnt=0;//LED soft PWM variable
// intensity look-up table LED-PWM:
byte Ledtab[32]={0,10,20,40,60,80,110,130, 150,175,190,200,207,214,220,225, 
230,233,236,238,240,242,244,245, 246,247,248,249,250,251,252,253};

void setup(){
Serial.begin(31250);// MIDI Baudrate
pinMode(2, INPUT_PULLUP); //pushbutton trigger
pinMode(3, OUTPUT); //PWM output for pan voltage
pinMode(4, OUTPUT); //LED Filtermode LP
pinMode(5, OUTPUT); //LED Filtermode BP
pinMode(6, OUTPUT); //LED Filtermode HP
pinMode(7, OUTPUT); //LED LFO wave

DDRB=B00111111;
//init Timer 1, PRBS noise generator and software filters clock oscillator
TCCR1A=B00000001;   //COM1A1/COM1A0/COM1B1/COM1B0/-/-/WGM11/WGM10
TCCR1B=B00010001;   // x/x/x/WGM13/WGM12/CS12/CS11/CS10
//CSxxx:000=stop,001=div1,010=div8,011=div64,100=div256
TIMSK1=B00000010; // Interrupt on: OCRA,(OCRB,OCRA,OVERFLOW)
OCR1A =200;//200=40kHz for prescaler=1
    // Freq=16MHz/prescaler/(OCR1A+1) example:1381.2Hz for OCR1A=180 prescaler=64
    // prescaler word= CS22,CS21,CS20: 
    // 111=/1024, 110=/256, 101=/128, 100=/64. 011=/32, 010=/8, 001=/1, 000=stop

//init Timer 0 interrupt for softPWM LEDs, MIDI interpolator, loop parameter read and parameter update
TCCR0A=B00000000;//COM0A1/COM0A0/COM0B1/COM0B0/-/-/WGM01/WGM00
TCCR0B=B00000010;// FOC0A/FOC0B/x/x/WGM02/CS02/CS01/CS00
//CSxxx:000=stop,001=div1,010=div8,011=div64,100=div256
TIMSK0=B00000010; // Interrupt on: OCRA,(OCRB,OCRA,OVERFLOW)
// initialize Timer2 mode register for 1 channel PWM controlling PanPosition:
TIMSK2=B00000000;//no interrupts
TCCR2A=B00100011;//pwm fast
TCCR2B=B00000001;//prescaler=1, pwm=62k5Hz
OCR2B=127;//init set to midscale
}

void loop() {
//MIDI_Poll(); //check for MIDIdata
if (TickDiv > 1){ // update potmeters and modulators every 10ms  
   Tick=true;
   TickDiv=0;
}

if (Tick == true){//update modulators and read potm-parameters each tick

// read triggerswitch
LAST_PBtn1=PBtn1;
PBtn1=digitalRead(2);

//read analog potmeters and check if the value has changed, if so: override possible MIDI set value
// disable this block for MIDI only, if you have no potmeters connected
// /*
//aPotmNc=analogRead(A6)>>3;//read and scale to 0-127
//if (aPotmNc != LAST_aPotmNc){PotmNc=aPotmNc;LAST_aPotmNc=aPotmNc;}//check if there is a change in value

aPotmFf=analogRead(A5)>>3;//scale to 0-127
if (aPotmFf != LAST_aPotmFf){PotmFf=aPotmFf;LAST_aPotmFf=aPotmFf;}
aPotmFt=analogRead(A0)>>3;
if (aPotmFt != LAST_aPotmFt){PotmFt=aPotmFt;LAST_aPotmFt=aPotmFt;}
aPotmMf=analogRead(A2)>>3;//scale to 0-127
if (aPotmMf != LAST_aPotmMf){PotmMf=aPotmMf;LAST_aPotmMf=aPotmMf;}
aPotmMa=analogRead(A1)>>3;//scale to 0-127
if (aPotmMa != LAST_aPotmMa){PotmMa=aPotmMa;LAST_aPotmMa=aPotmMa;}
aPotmMw=analogRead(A4)>>3;//scale to 0-127
if (aPotmMw != LAST_aPotmMw){PotmMw=aPotmMw;LAST_aPotmMw=aPotmMw;}
aPotmMp=analogRead(A3)>>3;//scale to 0-127
if (aPotmMp != LAST_aPotmMp){PotmMp=aPotmMp;LAST_aPotmMp=aPotmMp;}
//aPotmPn=analogRead(A5)>>3;//scale to 0-127
//if (aPotmPn != LAST_aPotmPn){PotmPn=aPotmPn;LAST_aPotmPn=aPotmPn;}
// */
//end of read analog potmeters

//determine Filter type:
Ft=PotmFt/8;//16 filtertypes
if (Ft != LAST_Ft){// if filtertype is changed: clear filter registers to prevent overload
//  F1=15;
//  DAC=0;
  LP=0;
  BP=0;
  HP=0;
  LAST_Ft=Ft;
}

Nmode=PotmNc>>3;//define 16 steps Noise mode

if (Nmode<5){//if in noise mode
    if ((LAST_PWH>>1) +LAST_NOTE+aPotmMf >127){PotmMf=127;}//limit high to 127
    else if ((LAST_PWH>>1) +LAST_NOTE+aPotmMf <0){PotmMf=0;}// limit low to 0
    else {PotmMf=(LAST_PWH>>1) +LAST_NOTE+aPotmMf;}  
    if (Nmode==3){Nlim=(38-PotmNc)<<2;}//noise mode crisp 1 (water)
    else if (Nmode==4){Nlim=(40-PotmNc)<<1;}//noise mode crisp 2 (crackle)
}

else if (Nmode >13){//if in pattern mode LFO sweep
  if (PotmMa<63){//if LFO is doing filtermodulation
    LenMod=((PotmNc-110)<<2)+(E1Fval>>2)+LAST_NOTE;
    if (LenMod >127){LenMod=127;}
  }
   else{//if LFO is doing amplitude modulation 
     LenMod=(((PotmMa-65)*E1val)>>7)+LAST_NOTE;
     if (LenMod >127){LenMod=127;}
   }
  NoiseLen=pgm_read_word_near(NoiseLenTab+LenMod);
}

else if (Nmode>4){//if in pattern mode potmeter/MIDI set
  LenMod=(LAST_NOTE+PotmNc-16)+(LAST_PWH>>2);
  if (LenMod >127){LenMod=127;}
  NoiseLen=pgm_read_word_near(NoiseLenTab+LenMod);
}

//define LFO mode
if (PotmMp<1) {E1mode=2;}
else if (PotmMp<96) {E1mode=3;}
else if (PotmMp<127 ) {E1mode=4;}
else {E1mode=0;}//single trigger mode

// LFO handling:
if ((E1cnt>32767) || (KEY_START==true)){// if waveform positioncounter is max or key started
  E1cnt=0;//reset LFO wave
  if ((PotmMa>80) &&((PotmMw>>3)<8)){//clear filter register on amplitude control mode LFO to prevent click
    LP=0;
  }
 Rn1=(Rn1<<1 | (((Rn1>>7)&1) ^ ((Rn1>>5)&1) ^ ((Rn1>>4)&1) ^ ((Rn1>>3)&1)) );// new random number
 Rncnt++;//increase LFO random number counter
 Cyclecnt++;//increase LFO cycle counter
 CycleLen=pgm_read_word_near(PatLenTab+(PotmMp>>1));
 if ((Cyclecnt >= CycleLen )||(KEY_START==true)){Cyclecnt=0;}
 if (E1mode>0 && KEY_DOWN==true){E1Tr=HIGH;}
 else {E1Tr=LOW;}// reset trigger
}

if (E1mode==0){//LFO mode: single trigger on pushbutton
  if (PBtn1==LOW  && LAST_PBtn1==HIGH){
     E1Tr=HIGH;
     E1cnt=0;
  }
  else if (KEY_START==true){
     E1Tr=HIGH;
     E1cnt=0;
     LP=0;
  }
}

if (E1Tr==HIGH){
  switch (PotmMw>>3) { //select LFO waveform 0-15 and calculate actual value
   case 0:// sawdown short 
     if (E1cnt <6144){  E1val=255-(E1cnt>>5);}
     else {E1val=56 -((E1cnt-4096)>>9);}
   break;
   case 1://sawdown exp
      if (E1cnt <4096){E1val=255-(E1cnt>>5);}
      else {E1val=118-((E1cnt-4096)>>8);}
   break;
   case 2:// sawdown lin 
      E1val=255-(E1cnt>>7);
   break;
   case 3://sawup4down28
      if (E1cnt <4096){E1val=(E1cnt>>4);}
      else{E1val=255-((E1cnt-2096)>>7);}
   break;
   case 4://sawup16down16, triangular
      if (E1cnt <16384){E1val=E1cnt>>6; }
      else{E1val=(16384-E1cnt)>>6;}
   break;
   case 5://sawup28down4
      if (E1cnt <28672){E1val=(E1cnt>>7)+(E1cnt>>10);}
      else{E1val=255-((E1cnt-28672)>>4);}
   break;
   case 6://sawup
      E1val=E1cnt>>7;    
   break;
   case 7://square 50% smooth edge
     if (E1cnt < 512){E1val=E1cnt>>1;}
     else if (E1cnt <16384){E1val=255;}
     else if (E1cnt <16896){E1val=(16896-E1cnt)>>1;}
     else {E1val=0;}
   break;
   case 8://square 25% + step down at 50%
      if (E1cnt < 8192){E1val=255;}
      else if (E1cnt < 16384){E1val=96;}
      else {E1val=0;}
   break;
   case 9://square 25%
      if (E1cnt < 8192){E1val=255;}
      else {E1val=0;}
   break;
   case 10://square 6%
       if (E1cnt < 64){E1val=255;}
      else {E1val=0;}
   break;
   case 11:// random step +  Return To Zero level
      if (E1cnt < 16384){E1val=Rn1;}
      else {E1val=0;}//E1val= Rn1;
   break;  
    case 12 :// random step + Return To Half level
      if (E1cnt < 16384){E1val=Rn1;}
      else {E1val=Rn1>>1;}//E1val= Rn1;
   break;  
    case 13://random step 
      E1val=Rn1;
   break;
   case 14://random step plus smoothing lowpass
       E1val= Rn1_F-(Rn1_F>>2) +(Rn1>>2);
       Rn1_F=E1val;
   break;
   case 15://random step plus leaky integrator
      Rn1_F= (Rn1_F)-(Rn1_F>>7)+(Rn1-128);//lowpassfilter
      E1val= 128+(Rn1_F>>7);
   break;
  }//end off switch waveform
}//end off if E1Tr=HIGH


if (E1mode==2){//LFO mode is in continuous mode
 // E1Accent=(pgm_read_word_near(PatTab+(PotmMp>>1)) << Cyclecnt)&128; // get an accent-enable-bit from Pattern table
  if (PBtn1==LOW){
    E1val=E1val>>1;//lower LFO value on accent=0
  }
}

else if (E1mode==3){//LFO mode is in pattern-accent mode
  E1Accent=(pgm_read_word_near(PatTab+(PotmMp>>1)) << Cyclecnt)&128; // get an accent-enable-bit from Pattern table
  if (E1Accent == 0  && PBtn1==HIGH){
    E1val=E1val>>1;//lower LFO value on accent=0
  }
}
else if(E1mode==4){//LFO mode is in pattern-sweep mode
  E1Accent=pgm_read_word_near(Pat2Tab+Cyclecnt+((PotmMp-96)<<3));  
  E1val=(E1Accent*E1val)>>8;
}  

if (PotmMa<63){//LFO does filter modulation
  PotmAm=(127*MidiVol)>>7;
  E1Fval=(E1val*(62-PotmMa))>>6;
  E1Fval=(LAST_VEL*E1Fval)>>7;
  if ((PotmFf<<1)+E1Fval>255){F1=255;}
  else {F1=(PotmFf<<1)+E1Fval;}
}
else if (PotmMa<67){//LFO is off, if PAN-LFO is also off
  PotmAm=(127*MidiVol)>>7;
  //if (PotmPn>47){E1val=0;}
  E1val=0;
  F1=PotmFf<<1;
}
else{//LFO does amplitude modulation
  F1=PotmFf<<1;
  PotmAm=(((PotmMa-66)*E1val)>>7)+(255-(PotmMa<<1));
  PotmAm=(MidiVol*PotmAm)>>7;
  E1Fval=0;
}

switch (Ft){//set for each filtertype: low-limit, scale, dc-offset, output amplitude (Amp), whitenoise amplitude, filtermode LEDs 
case 0:// 1st order lowpass
  OFS=0;// dc-offset correctie for filter output
  WNamp=40;//scale white noise amplitude
  Amp=(PotmAm<<1)-(PotmAm>>1);//scale output amplitude
  LEDFm=B00010000;//enable LED LP
break;
case 1:// 2nd order lowpass L Q
  OFS=0;
  if (F1<10){F1=10;} 
  WNamp=50-(F1>>3);
  Amp=(PotmAm<<1)-(PotmAm>>1);
  LEDFm=B00110000;//enable LED LP+BP
break;
case 2:// 2nd order lowpass M  Q
  OFS=0;
  if (F1<10){F1=10;}
  WNamp=45-(F1>>3); 
  Amp=(PotmAm<<1)-(PotmAm>>1);
  LEDFm=B00110000;//enable LED LP+BP
break;
case 3:// 2nd order lowpass H Q
  OFS=0;
  if (F1<10){F1=10;}
  WNamp=45-(F1>>3); 
  Amp=(PotmAm<<1)-(PotmAm>>1);
  LEDFm=B00110000;//enable LED LP+BP
break;
case 4:// bandpass L Q
  if (F1<30 && Nmode<2){OFS=F1-29;}
  else {OFS=0;}
  WNamp=26-(F1>>4);
  if (F1<6){F1=6;}
  else {F1= 5+((F1 *85)>>7);}
  Amp=(PotmAm<<1)-(PotmAm>>1);
  LEDFm=B00100000;//enable LED BP
break;
case 5:// bandpass ML Q
  if (F1<30 && Nmode<2){OFS=F1-29;}
  else {OFS=0;}
  WNamp=35-(F1>>4);
  if (F1<8){F1=8;}
  Amp=PotmAm;
  LEDFm=B00100000;//enable LED BP
break;
case 6:// bandpass M Q
  if (F1<30 && Nmode<2){OFS=F1-29;}
  else {OFS=0;}
  WNamp=34-(F1>>4);
  Amp=PotmAm;
  LEDFm=B00100000;//enable LED BP
  if (F1<10){F1=10;}
break;
case 7:// bandpass MH Q
  if (F1<30 && Nmode<2){OFS=F1-29;}
  else {OFS=0;}
  WNamp=30-(F1>>4);
  if (F1<10){F1=10;}
  Amp=PotmAm;
  LEDFm=B00100000;//enable LED BP
break;
case 8:// bandpass H Q
  if (F1<30 && Nmode<2){OFS=F1-29;}
  else {OFS=0;}
  WNamp=28-(F1>>5);//45
  if (F1<10){F1=10;}
  Amp=PotmAm;
  LEDFm=B00100000;//enable LED BP
break;
case 9:// bandpass HH Q
  if (F1<4){F1=4;}
  if (F1<30 && Nmode<2){OFS=F1-29;}
  else {
    if (F1>240){F1=240;}
    OFS=0;
  }
   WNamp=28-(F1>>5);//45
  Amp=PotmAm;
  LEDFm=B00100000;
break;
case 10://highpass 1st order
  OFS=0;
  WNamp=40;
  Amp=(PotmAm<<1)-(PotmAm>>1);
  LEDFm=B01000000;//enable LED HP
break;
case 11:// highpass low Q
  OFS=0;
  if (F1<10){F1=10;}
  WNamp=35-(F1>>4);
  Amp=(PotmAm<<1)-(PotmAm>>1)-(PotmAm>>2);
  LEDFm=B01100000;//enable LED BP+HP
break;
case 12:// highpass high Q
  if (F1<40){OFS=F1-39;}
  else {OFS=0;}
  if (F1<8){F1=8;}
  WNamp=35-(F1>>4);
  Amp=(PotmAm<<1)-(PotmAm>>1)-(PotmAm>>2);
  LEDFm=B01100000;//enable LED BP+HP
break;
case 13:// bandreject low Q
  if (F1<35 && Nmode<3){OFS=34-F1;}
  else {OFS=0;}
  WNamp=40-(F1>>3);
  if (F1<4){F1=4;}
  else{F1= (F1 *85)>>7;}
  Amp=(PotmAm<<1)-(PotmAm>>1)-(PotmAm>>2);
  LEDFm=B01010000;//enable LED LP+HP
break;
case 14:// bandreject med Q
  OFS=0;
  WNamp=26;
  Amp=(PotmAm<<1)-(PotmAm>>1)-(PotmAm>>2);
  LEDFm=B01010000;//enable LED LP+HP
break;
case 15:// varable clock prbs
  OFS=0;
  WNamp=35;
  Amp=PotmAm;
  LEDFm=B01110000;
break;
}// end of switch filtertype

if (KEY_DOWN==true){DACAmp=Amp;}//if key down, amplitude normal 
//else if (DACAmp>0){DACAmp=DACAmp-1;}//amplitude decay on key up
else if (DACAmp>0){DACAmp=(DACAmp*63)>>6;}//amplitude decay on key up

// pan handling, using 62kHz PWM via timer 0:
if (PotmPn >79){//potm at right part of scale, dedicated LFO, variable speed
 if (E2cnt>32767){E2cnt=0;}// if waveform positioncounter is max clear cnt
 if (E2cnt <16384){E2val=E2cnt>>6; }//make triangular wave
      else{E2val=(16383-E2cnt)>>6;}
E2cnt=E2cnt+ pgm_read_word_near(TkUpd+(PotmPn<<1)-144);
OCR2B=E2val;
}
else if (PotmPn <48){//potm at left part of scale, use main LFO

 if (PotmPn >40){
  if (E2cnt>32767){E2cnt=0;}// if waveform positioncounter is max clear cnt
  if (E2cnt <16384){E2val=E2cnt>>6; }//make triangular wave
      else{E2val=(16383-E2cnt)>>6;}
  E2cnt=E2cnt+ (pgm_read_word_near(TkUpd+PotmMf)>>4);
  OCR2B=E2val;
 }

 else if (PotmPn >32){
  if (E2cnt>32767){E2cnt=0;}// if waveform positioncounter is max clear cnt
  if (E2cnt <16384){E2val=E2cnt>>6; }//make triangular wave
      else{E2val=(16383-E2cnt)>>6;}
  E2cnt=E2cnt+ (pgm_read_word_near(TkUpd+PotmMf)>>3);
  OCR2B=E2val;
 }
 else if (PotmPn >24){
  if (E2cnt>32767){E2cnt=0;}// if waveform positioncounter is max clear cnt
  if (E2cnt <16384){E2val=E2cnt>>6; }//make triangular wave
      else{E2val=(16383-E2cnt)>>6;}
  E2cnt=E2cnt+ (pgm_read_word_near(TkUpd+PotmMf)>>2);
  OCR2B=E2val;
 }
 else if (PotmPn >16){
  if (E2cnt>32767){E2cnt=0;}// if waveform positioncounter is max clear cnt
  if (E2cnt <16384){E2val=E2cnt>>6; }//make triangular wave
      else{E2val=(16383-E2cnt)>>6;}
  E2cnt=E2cnt+ (pgm_read_word_near(TkUpd+PotmMf)>>1);
  OCR2B=E2val;
 }
 else if (PotmPn >8){
  if (E2cnt>32767){E2cnt=0;}// if waveform positioncounter is max clear cnt
  if (E2cnt <16384){E2val=E2cnt>>6; }//make triangular wave
      else{E2val=(16383-E2cnt)>>6;}
   E2cnt=E1cnt;
  OCR2B=E2val;
 }
 else{
  if (E1cnt<24484){E2cnt=E1cnt+8192;}
  else {E2cnt=E1cnt-24484;}
  if (E2cnt <16384){E2val=E2cnt>>6; }//make triangular wave
  else{E2val=(16383-E2cnt)>>6;}
  OCR2B=E2val;
 }
}//end of if potm<48, potm at left part of scale, use main LFO

else{// middle part of knob scale: static pan set by pot
  OCR2B=pgm_read_word_near(PanTab+(79-PotmPn));
  E2cnt=0;
}
// end of pan handling

//set sampling clock rate= timer 1 rate:
if (Ft>14){//if filter type set to samplingclock rate modulation 
Clk=200+((255-F1)<<4);
}
else{
  if (Nmode >4 ){Clk=213;}// tune this for musical pitch offset in pattern mode
  else{Clk=225 +(255-F1);}//can be as low as Clk=190 for IDE compiler 1.04 because of faster output code
}
  noInterrupts();
  OCR1A =Clk;//200=40kHz for prescaler=1
  interrupts(); 
// end of set sampling  rate

E1cnt=E1cnt+ pgm_read_word_near(TkUpd+PotmMf);// get LFO counter increment value, set by potm 
Tick=false;
KEY_START=false;
 }//end of if tick is true, update modulators and read potm-parameters each tick
}//end of main loop


//interupt handler timer 0
ISR(TIMER0_COMPA_vect)// update tick for LEDs, modulators+parameter update
{

// LED indicators handling, soft PWM -------------------------------------------
if (LedPWMcnt >30){
  LedPWMcnt=0;
  TickDiv++;
}
else {LedPWMcnt++;}//softPWM counter runs from 0 to 31
if (Ledtab[LedPWMcnt] >F1 ){PORTD = PORTD & B10001111;} //LEDs off
else {PORTD = PORTD | LEDFm;}// LEDs showing filtermode on
if (Ledtab[LedPWMcnt] >= E1val){PORTD = PORTD & B01111111;} //LED LFO off
else {PORTD = PORTD | B10000000;}// LED LFO on

//end of LED indicators handling ---------------------------------------------------

}// end of interupt handler timer 0,  update tick for modulators+parameter update


//interupt handler for timer 1, audio sample rate 
ISR(TIMER1_COMPA_vect)//clock for audio noisesource, noisefilters and audio R-DAC output
{
// concept: 16 bit PRBS with shift register and XOR on Bit 16,15,13,4  makes white noise
switch (Nmode){//select on noise mode
case 0://red noise
  s2cnt++; // add low frequency noise F/s2
  s3cnt++;// add low frequency noise F/s3
  s4cnt++;// add low frequency noise F/s4
  // use lowest frequency priority to have less computational load
  if (s4cnt>26){// make F/27 noise
    state4=(state4<<1 | (((state4>>15)&1) ^((state4>>14)&1) ^ ((state4>>12)&1) ^ ((state4>>3)&1) ));
    s4cnt=0;
  }
  else if (s3cnt>8){// make  F/9 noise
    state3=(state3<<1 | (((state3>>15)&1) ^((state3>>14)&1) ^ ((state3>>12)&1) ^ ((state3>>3)&1) ));
    s3cnt=0;
  }
  else if (s2cnt>2){// make F/3 noise
    state2=(state2<<1 | (((state2>>15)&1) ^((state2>>14)&1) ^ ((state2>>12)&1) ^ ((state2>>3)&1) ));
    s2cnt=0;
  }
  else{// mak F noise
    state1=(state1<<1 | (((state1>>15)&1) ^((state1>>14)&1) ^ ((state1>>12)&1) ^ ((state1>>3)&1) ));  
  }
  DAC=(state1 & 16)+(state2>>11)+(state3>>11)+(state4>>10)-68;//16 bit length 7 bit value 1/F^2 noise, dc removed
break;

case 1://pink noise
  s2cnt++;// add low frequency noise at rate F/s2
  s3cnt++;// add low frequency noise at rate F/s3
  s4cnt++;// add low frequency noise at rate F/s4

  if (s4cnt>28){
    state4=(state4<<1 | (((state4>>15)&1) ^((state4>>14)&1) ^ ((state4>>12)&1) ^ ((state4>>3)&1) ));
    s4cnt=0;
  }
  else if (s3cnt>8){
    state3=(state3<<1 | (((state3>>15)&1) ^((state3>>14)&1) ^ ((state3>>12)&1) ^ ((state3>>3)&1) ));
    s3cnt=0;
  }
  else if (s2cnt>2){
    state2=(state2<<1 | (((state2>>15)&1) ^((state2>>14)&1) ^ ((state2>>12)&1) ^ ((state2>>3)&1) ));
    s2cnt=0;
  }
  else{
    state1=(state1<<1 | (((state1>>15)&1) ^((state1>>14)&1) ^ ((state1>>12)&1) ^ ((state1>>3)&1) ));  
  }
  DAC=(state1 & 32)+(state2>>10)+(state3>>12)+(state4>>12)-65;//16 bit length 7 bit value 1/F noise, dc removed
break;

case 2://white noise
  s2cnt++;// use a second PRBS to interleave, this prevents hearing a repetitive 1.5 second repeat pattern
  if (s2cnt>2){
    state2=(state2<<1 | (((state2>>15)&1) ^((state2>>14)&1) ^ ((state2>>12)&1) ^ ((state2>>3)&1) ));
    s2cnt=0;
    DAC=(state2 & 64);
  }
  else{
    state1=(state1<<1 | (((state1>>15)&1) ^((state1>>14)&1) ^ ((state1>>12)&1) ^ ((state1>>3)&1) ));  
    DAC=(state1 & 64);
  }
  if (DAC>0){DAC=WNamp;}//set amplitude
  else{DAC=-WNamp;}
break;

case 3://crisp noise 1 (water)
  s2cnt++;
  s3cnt++;
  s4cnt++;

  if (s4cnt>26){
    state4=(state4<<1 | (((state4>>15)&1) ^((state4>>14)&1) ^ ((state4>>12)&1) ^ ((state4>>3)&1) ));
    s4cnt=0;
  }
  else if (s3cnt>9){
    state3=(state3<<1 | (((state3>>15)&1) ^((state3>>14)&1) ^ ((state3>>12)&1) ^ ((state3>>3)&1) ));
    s3cnt=0;
  }
  else if (s2cnt>3){
    state2=(state2<<1 | (((state2>>15)&1) ^((state2>>14)&1) ^ ((state2>>12)&1) ^ ((state2>>3)&1) ));
    s2cnt=0;
  }
  else{
    state1=(state1<<1 | (((state1>>15)&1) ^((state1>>14)&1) ^ ((state1>>12)&1) ^ ((state1>>3)&1) ));  
  }
  DAC=(state1 & 16)+(state2>>12)+(state3>>11)+(state4>>11);
  if (DAC>Nlim){DAC=DAC-55;}//make waterstream noise 20-55 Nlim set by potm, tunes density
break;

case 4://crisp noise 2 (crackle)
  s2cnt++;
  s3cnt++;
  s4cnt++;

  if (s4cnt>80){//80
    state4=(state4<<1 | (((state4>>15)&1) ^((state4>>14)&1) ^ ((state4>>12)&1) ^ ((state4>>3)&1) ));
    s4cnt=0;
  }
  else if (s3cnt>16){//16
    state3=(state3<<1 | (((state3>>15)&1) ^((state3>>14)&1) ^ ((state3>>12)&1) ^ ((state3>>3)&1) ));
    s3cnt=0;
  }
  else if (s2cnt>2){//2
    state2=(state2<<1 | (((state2>>15)&1) ^((state2>>14)&1) ^ ((state2>>12)&1) ^ ((state2>>3)&1) ));
    s2cnt=0;
  }
  else{
    state1=(state1<<1 | (((state1>>15)&1) ^((state1>>14)&1) ^ ((state1>>12)&1) ^ ((state1>>3)&1) ));  
  }
  DAC=(state1 & 32)+(state2>>12)+(state3>>12)+(state4>>12);
  if (DAC>Nlim){//Nlim set by potm, tunes crackle density
  DAC=32+(DAC>>2);}
break;

default://variable length PRBS 
  state1=(state1<<1 | (((state1>>15)&1) ^((state1>>14)&1) ^ ((state1>>12)&1) ^ ((state1>>3)&1) )); 
   if (NoiseCnt >= NoiseLen){
     state1=1+Rncnt;
     NoiseCnt=0;
   }
  else {NoiseCnt++;}
  DAC=(state1& 63)-32;
break;
}//end off select noise type

switch (Ft){//select filtertype 
case 0://1st order lowpass
  LP=LP+(DAC-(LP>>8))*F1;
  DAC=(LP>>8);//lowpass  
break;

case 1://2nd order lowpass L Q
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - BP;
  BP = ((F1 * HP)>>8) + BP;
  DAC= LP ;
break;

case 2://2nd order lowpass M Q
  DAC=(DAC>>1)+(DAC>>2);
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - (BP>>1);
  BP = ((F1 * HP)>>8) + BP;
  DAC=LP ;
break;

case 3://2nd order lowpass H Q
  DAC=(DAC>>1)+(DAC>>4);
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - (BP>>2);
  BP = ((F1 * HP)>>8) + BP;
  DAC=LP ;
break;

case 4://bandpass L Q
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - (BP<<1);
  BP = ((F1 * HP)>>8) + BP;
  DAC=BP<<1;
break;

case 5:// bandpass ML Q
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - BP;
  BP = ((F1 * HP)>>8) + BP;
  DAC=BP<<1 ;
break;

case 6://bandpass M Q
  DAC=(DAC>>1)+(DAC>>2);
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - (BP>>1);
  BP = ((F1 * HP)>>8) + BP;
  DAC=BP<<1 ;
break;

case 7://bandpass MH Q
  DAC=DAC>>1;
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - (BP>>2);
  BP = ((F1 * HP)>>8) + BP;
  DAC=BP<<1;
break;

case 8://bandpass H Q
  DAC=(DAC>>2);
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - (BP>>3);
  BP = ((F1 * HP)>>8) + BP;
  DAC=BP<<1 ;
break;

case 9://bandpass HH Q
  LP = LP + ((F1 * BP)>>8);
  HP = (DAC>>2) - LP - (BP>>5);
  BP = ((F1 * HP)>>8) + BP;
  DAC= BP;
break;

case 10://1st order highpass
  LP=LP+(DAC-(LP>>8))*F1;
  DAC=DAC-(LP>>8);//highpass 
break;

case 11://2nd order highpass L Q
  DAC=DAC>>1;
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - BP;
  BP = ((F1 * HP)>>8) + BP;
  DAC=HP<<1 ;
break;

case 12://2nd order highpass H Q
  DAC=DAC>>1;
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - (BP>>1);
  BP = ((F1 * HP)>>8) + BP;
  DAC=HP<<1 ;
break;

case 13://2nd order bandstop L Q
  DAC=(DAC>>1)+(DAC>>2);
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - (BP<<1);
  BP = ((F1 * HP)>>8) + BP;
  DAC=LP+HP ;
break;

case 14://2nd order bandstop H Q
  LP = LP + ((F1 * BP)>>8);
  HP = DAC - LP - (BP>>1);
  BP = ((F1 * HP)>>8) + BP;
  DAC=LP+HP ;
break;

case 15://fixed filter, variable clock prbs
  LP = LP + ((160 * BP)>>8);
  HP = DAC - LP - BP;
  BP = ((160 * HP)>>8) + BP;
  DAC= LP ;
break;

}// end off select filtertype

// calculate and write DAC value:
DAC=DAC+OFS;//trim offset for each filter type and setting
DAC=(DAC* DACAmp)>>7;//set amplitude
// write value to BiTri DAC, 6 pins of port B
DDRB=pgm_read_byte_near(TDACD+DAC+128);//set data direction for trinairy
PORTB=pgm_read_byte_near(TDACV+DAC+128);//set value
//end of calculate and write DAC value

}//end of interupt handler for timer 1, audio rate//interupt handler for timer 1, audio rate


//MIDI receive subroutine:
 void MIDI_Poll(){
   if (Serial.available() > 0) {
   do {
       incomingByte = Serial.read(); // read the incoming byte:
       if (incomingByte>247) {       // these are system real-time messages
         if (incomingByte == 248) {  //a midiclock byte
        // MidiClk=true; // enable MIDIclock sync
        //MidiTick=true; // set MIDIclock event flag
        // TickDiv=0;// reset ext clock time-out timer
         }
       }
         else if (incomingByte>240) { //these are system exclusive data messages
          statusbuffer = 0;
         }
         else if (incomingByte>127) { //these are channel voice messages
           statusbuffer = incomingByte;
           firstbyte = true;
           statusByte = 0;
           dataByte = 0;
         }
         else if (statusbuffer!=0) {
         if (firstbyte == true) {// must be first byte
           statusByte = incomingByte;
           firstbyte = false;
            if (statusbuffer == PROG_CHANGE){ //program change message
             LAST_PROG = statusByte;
            }
           if (statusbuffer == AFT_CHANGE){LAST_AFT = statusByte;} //channel aftertouch message
         } //end of: if first byte is true
         else { // it is second byte
           dataByte = incomingByte;//process the message
           if (statusbuffer == NOTE_ON && dataByte != 0) { //if MIDI note_on message with velocity >0
             if (KEY_DOWN == false){KEY_START =true;}//only retrigger if not playing notes legato
             KEY_DOWN = true;
             LAST_NOTE = statusByte;
             LAST_VEL = dataByte;
             MidiCtrl=true;// set remote control flag
           }
             else if (statusbuffer == NOTE_OFF || (statusbuffer == NOTE_ON && dataByte == 0)) {
               //if MIDI note_off message or equivalent message: note_on with velocity=0
               if (statusByte == LAST_NOTE){KEY_DOWN = false;}
            }
           else if (statusbuffer == PITCHW_CHANGE){LAST_PWH = dataByte-64;} //pitch bend wheel message                     
           else if (statusbuffer == CONT_CHANGE){  // other controllers:
           MidiCtrl=true;// set remote control flag
           //todo replace by switch case?:
          //switch (statusByte){
        //case 1:
        //PotmFf = dataByte;
        //break;
        //} //end of switch case statusbyte
            if (statusByte==1) {PotmFf = dataByte;}//Modulation Wheel doing filter frequency
            else if (statusByte==7) {MidiVol = dataByte;}//main volume
            else if (statusByte==73) {PotmFt = dataByte;}//Filter type
            else if (statusByte==77) {PotmMf = dataByte;}//LFO rate
            else if (statusByte==76) {PotmMa = dataByte;}//LFO depth
            else if (statusByte==78) {PotmMw = dataByte;}//LFO wave 
            else if (statusByte==79) {PotmMp = dataByte;}//LFO mode
            else if (statusByte==10) {PotmPn = dataByte;}//Pan
            else if (statusByte==75) {PotmNc = dataByte;}//Noise Color 
          }//end of other controllers
          //clear variables for next message:
          statusByte = 0;
          dataByte = 0;
          firstbyte = true;        
         }
       }
     }
  while (Serial.available() > 0); //check for more data
   }
}//end of MIDI receive routine
